package at.ac.uibk.gitsearch.service.management;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.actuate.endpoint.annotation.Endpoint;
import org.springframework.boot.actuate.endpoint.annotation.ReadOperation;
import org.springframework.boot.actuate.health.Health;
import org.springframework.boot.actuate.health.HealthIndicator;
import org.springframework.boot.actuate.health.Status;
import org.springframework.stereotype.Service;

import at.ac.uibk.gitsearch.config.ApplicationProperties;
import at.ac.uibk.gitsearch.service.PluginManagementService;
import at.ac.uibk.gitsearch.service.PluginManagementService.PluginConfigWrapper;

@Service
//@Endpoint(id = "pluginHealth")
public class PluginHealthCheck implements HealthIndicator {

	@Autowired
	protected PluginManagementService pluginManagementService;
	@Autowired 
	private ApplicationProperties applicationProperties;

	@Override
	public Health health() {
		
		Map<String, String> visited = new HashMap<>();
		 
		 for(PluginConfigWrapper pluginConfig: pluginManagementService.getRegisteredPluginConfigs()) {
			 final String message = String.format("Running as %s with %d actions", pluginConfig.getPluginName(), pluginConfig.getActions().size());
			visited.put(pluginConfig.getConfigURL(), message);
		 }
		 boolean down = visited.size() < applicationProperties.getRegisteredPlugins().size(); // some missing?
		 
		 applicationProperties.getRegisteredPlugins().stream().filter(configURL -> !visited.containsKey(configURL))
		 	.forEach(configURL -> visited.put(configURL, String.format("seems to be down")));
		 
		 return (down?Health.down():Health.up()).withDetails(visited).build();
	}
	

}
